@@ -24,6 +24,7 @@ public class DecoderInstance extends SixModelObject {
     private List<ByteBuffer> toDecode;
     private List<CharBuffer> decoded;
     private List<String> lineSeps;
+    private boolean translate_newlines = false;
 
     public void configure(ThreadContext tc, String encoding, SixModelObject config) {
         if (decoder == null) {
@@ -33,6 +34,8 @@ public class DecoderInstance extends SixModelObject {
             lineSeps = new ArrayList<String>();
             lineSeps.add("\n");
             lineSeps.add("\r\n");
+            if (config.exists_key(tc, "translate_newlines") != 0)
+                translate_newlines = (long) config.at_key_boxed(tc, "translate_newlines").get_int(tc) != 0;
         }
         else {
             throw ExceptionHandling.dieInternal(tc, "Decoder already configured");
@@ -66,6 +69,10 @@ public class DecoderInstance extends SixModelObject {
         }
     }
 
+    public synchronized String maybe_translate_newlines(ThreadContext tc, String str) {
+        return translate_newlines ? str.replace("\r\n", "\n") : str;
+    }
+
     public synchronized String takeChars(ThreadContext tc, long chars, boolean eof) {
         ensureConfigured(tc);
 
@@ -89,6 +96,7 @@ public class DecoderInstance extends SixModelObject {
         String normalized = Normalizer.normalize(
             decodedBuffer(target),
             Normalizer.Form.NFC);
+        normalized = maybe_translate_newlines(tc, normalized);
         if (normalized.length() > chars) {
             String result = normalized.substring(0, (int)chars);
             String remaining = normalized.substring((int)chars, normalized.length());
@@ -128,6 +136,7 @@ public class DecoderInstance extends SixModelObject {
         String normalized = Normalizer.normalize(
             decodedBuffer(target),
             Normalizer.Form.NFC);
+        normalized = maybe_translate_newlines(tc, normalized);
         if (normalized.length() == 0 || isNormTerminated(normalized))
             return normalized;
         String result = normalized.substring(0, normalized.length() - 1);
@@ -156,7 +165,8 @@ public class DecoderInstance extends SixModelObject {
             decoder.flush(target);
             decoder.reset();
         }
-        return Normalizer.normalize(decodedBuffer(target), Normalizer.Form.NFC);
+        String normalized = Normalizer.normalize(decodedBuffer(target), Normalizer.Form.NFC);
+        return maybe_translate_newlines(tc, normalized);
     }
 
     public synchronized String takeLine(ThreadContext tc, boolean chomp, boolean eof) {
@@ -191,10 +201,13 @@ public class DecoderInstance extends SixModelObject {
             CharBuffer target = CharBuffer.allocate(decodee.limit());
 
             CoderResult result = decoder.decode(decodee, target, eof && toDecode.size() == 1);
-            /* TODO It looks like we read binary data with UTF_8 during
-             * normal operation; don't die then. */
-            if (result.isMalformed() && charset != StandardCharsets.UTF_8)
-                Ops.die_s("Will not decode invalid " + charset, tc);
+            if (result.isMalformed()) {
+                /* We encountered malformed input. But it could be that we found enough
+                 * valid input to return a whole line. So we should only die if we
+                 * didn't get anything in the output buffer. */
+                if (target.position() == 0)
+                    Ops.die_s("Will not decode invalid " + charset, tc);
+            }
 
             decoded.add(decodedBuffer(target));
             if (decodee.remaining() == 0)
