@@ -1,6 +1,7 @@
 package org.raku.nqp.sixmodel.reprs;
 
 import java.lang.System;
+import java.util.Arrays;
 
 import org.raku.nqp.runtime.ExceptionHandling;
 import org.raku.nqp.runtime.ThreadContext;
@@ -99,9 +100,7 @@ public class VMArrayInstance_u32 extends VMArrayInstanceBase {
             slots = new int[(int)ssize];
         }
         else {
-            int[] new_slots = new int[(int)ssize];
-            System.arraycopy(slots, 0, new_slots, 0, slots.length);
-            slots = new_slots;
+            slots = Arrays.copyOf(slots, (int)ssize);
         }
 
         this.slots = slots;
